import React, {useState, useEffect, useRef} from 'react';
import {
  Platform,
  StyleSheet,
  Text,
  View,
  TouchableOpacity,
  ScrollView,
} from 'react-native';
import RNCallKeep, {AnswerCallPayload, Events} from 'react-native-callkeep';

const EVENTS: Events[] = [
  'didReceiveStartCallAction',
  'answerCall',
  'endCall',
  'didActivateAudioSession',
  'didDeactivateAudioSession',
  'didDisplayIncomingCall',
  'didToggleHoldCallAction',
  'didPerformDTMFAction',
  'didResetProvider',
  'checkReachability',
  'didPerformSetMutedCallAction',
  'didLoadWithEvents',
  'showIncomingCallUi',
  'silenceIncomingCall',
];

type DidDisplayIncomingCallArgs = {
  error: string;
  callUUID: string;
  handle: string;
  localizedCallerName: string;
  hasVideo: string;
  fromPushKit: string;
  payload: string;
};

export default function App() {
  const [readyCK, setReadyCK] = useState(false);
  const [supportCS, setSupportCS] = useState(false);
  const [hasPhoneAccount, setHasPhoneAccount] = useState(false);

  const [callId, setCallId] = useState('');
  const [callerName, setCallerName] = useState('');
  const [callerId, setCallerId] = useState('');
  const isRinging = useRef(false);

  const removeEventListeners = () => {
    RNCallKeep.removeEventListener('endCall');
    RNCallKeep.removeEventListener('didDisplayIncomingCall');
  };

  const endCall = () => {
    RNCallKeep.endCall(callId);
    removeEventListeners();
  };

  const displayCallAndroid = () => {
    isRinging.current = true;
    RNCallKeep.displayIncomingCall(callId, callerName, callerName, 'generic');
    setTimeout(() => {
      if (isRinging.current) {
        isRinging.current = false;
        // 6 = MissedCall
        // https://github.com/react-native-webrtc/react-native-callkeep#constants
        RNCallKeep.reportEndCallWithUUID(callId, 6);
      }
    }, 15000);
  };

  const answerCall = ({callUUID}: AnswerCallPayload) => {
    isRinging.current = false;
    console.log('somewhere'); // navigated to call screen in our app
  };

  const didDisplayIncomingCall = (args: DidDisplayIncomingCallArgs) => {
    if (args.error) {
      console.log(`Callkeep didDisplayIncomingCall error: ${args.error}`);
    }

    isRinging.current = true;
    RNCallKeep.updateDisplay(callId, `${callerName}`, callerId);

    setTimeout(() => {
      if (isRinging.current) {
        isRinging.current = false;
        // 6 = MissedCall
        // https://github.com/react-native-webrtc/react-native-callkeep#constants
        RNCallKeep.reportEndCallWithUUID(callId, 6);
      }
    }, 15000);
  };

  const setupEventListeners = () => {
    RNCallKeep.addEventListener('endCall', endCall);
    RNCallKeep.addEventListener(
      'didDisplayIncomingCall',
      didDisplayIncomingCall,
    );
  };

  const initCK = async () => {
    try {
      RNCallKeep.setup({
        ios: {
          appName: 'WazoReactNativeDemo',
        },
        android: {
          alertTitle: 'Permissions required',
          alertDescription:
            'This application needs to access your phone accounts',
          cancelButton: 'Cancel',
          okButton: 'ok',
          selfManaged: true,
          additionalPermissions: [],
          foregroundService: {
            channelId: 'com.company.my',
            channelName: 'Foreground service for my app',
            notificationTitle: 'My app is running on background',
            notificationIcon: 'Path to the resource icon of the notification',
          },
        },
      });
      RNCallKeep.setAvailable(true);
      setupEventListeners();
      setSupportCS(RNCallKeep.supportConnectionService());
      const hasPA = await RNCallKeep.hasPhoneAccount();
      setHasPhoneAccount(hasPA);
      setReadyCK(true);
    } catch (err) {
      console.error('initializeCallKeep error:', (err as Error).message);
    }
  };

  useEffect(() => {
    initCK();
    return () => {
      removeEventListeners();
    };
  }, []);

  return (
    <View>
      <Text>CallKeep Demo: {readyCK ? 'Ready' : 'Waiting...'}</Text>
      <Text>Support Connection Service: {supportCS ? 'Yes' : 'No'}</Text>
      <Text>Has Phone account: {hasPhoneAccount ? 'Yes' : 'No'}</Text>
    </View>
  );
}
